
comment~
***********************************************************************

 Name:          detect.ASM

 Language:      TASM

 Author  :      

 Version History:

 Rev              Date                Description
 ----          ------------           ---------------------------------
 Created       Oct 14, 1991           Original
 Edited        Mar 12, 1992           ported code to seperate assy module


***********************************************************************

 Prologue:

			detect function

   Purpose:  * Detect the presence of a Cyrix Cx486SLC/DLC microprocessor.
	     * The undefined flags on a Cx486SLC/DLC remain unchanged 
	       following a divide.  The Intel part will modify the 
	       some of the undefined flags.  In this example the ZF flag 
	       should change.
	     * pseudocode
		save flags before
		load dividend and divisor
		do unsigned divide
		save flags after
		cmp flags before with flags after
		return a 1 if flags are unchanged ( Cyrix part detected )

   Called by:  main();

   Inputs:   none

   returns:  ax = 0 if Intel is detected
	     ax = 1 if Cyrix is detected

   ********************************************************************** */
~
       DOSSEG                       ;select Intel-convention
				    ;segment ordering
       .MODEL  SMALL                ;select small model
				    ;(nearcode and data)
       .DATA                        ;TC-compatible initialized
				    ;data segment

flags_mask      dw      08D5H   ; mask to isolate the undefined bits
				; masks all but OF,SF,ZF,AF,PF,CF flags
flags_before    DW      ?       ; flags before div
flags_after     DW      ?       ; flags after div
dividend        DW      0FFFFh  ; dividend
divisor         DW      4h      ; divisor
result          DW      0       ; results of flags compare
				; 0=different (not Cyrix), 1=same (Cyrix)

       .DATA?                       ;TC-compatible uninitialized
				    ;data segment
       .CODE                        ;TC-compatible code segment

       PUBLIC  _M5_detect

_M5_detect      PROC               ;function (near-callable in
				    ;small model)

      .286

      push bp                              ; "C" calling convention
      mov  bp,sp

      pusha                                ; save processor state
      pushf

					   ;set flags to a known value
      mov       ax,0
      cmp       ax,ax


      pushf                                ; load flags into ax
      pop   ax
      mov  ds:[ word ptr flags_before],ax  ; save flags to mem

; do a div instruction so that the signature of the undefined flags
; can be observed
      mov  ax, dividend
      mov  dx, 0
      mov  bx, divisor
      div  bx

      pushf                                     ;load flags into ax
      pop   ax
      mov   flags_after, ax                     ; save flags to mem

; recall flags_before and clear unwanted bits
      mov  ax, flags_mask
      and  ax, flags_before

; recall flags_after and clear unwanted bits
      mov  bx, flags_mask
      and  bx, flags_after

; compare the signature of the undefined bits before and after
      cmp  ax,bx
      jnz  Diff
      mov  result,1                     ; set if flag bits are unchanged
					; Cyrix part found
      jmp  Done

Diff: mov  result,0                     ; clear if flag bits are changed
					; Cyrix part not found

Done:
      popf                              ;restore processor state
      popa

      mov  ax,result                    ; return value in ax

      pop  bp                           ;"C" calling convetion
      ret

_M5_detect      ENDP

       END                              ;end detect.ASM module
